/*
 * A context resource offering support for opening files.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include "opener_context_object_interface.h"
#include "simple_pager.h"



/* Forward declaration. */

class OpenerResource;



/* Support for indicating files to be opened. */

class OpenerContextResource : public SimplePager, public OpenerContextObject
{
protected:
    OpenerResource *_opener;

public:
    explicit OpenerContextResource(OpenerResource *opener, Memory *memory=NULL);

    /* Server details. */

    int expected_items();

    ipc_server_handler_type handler();

    void *interface()
    { return static_cast<OpenerContextObject *>(this); }

    /* Data access methods. */

    char *get_path();

    /* Opener context interface methods. */

    long open(flags_t flags, offset_t *size, l4_cap_idx_t *file);

    /* Pager/dataspace methods. */

    long map(unsigned long offset, address_t hot_spot, flags_t flags,
             l4_snd_fpage_t *region)
    { return SimplePager::map(offset, hot_spot, flags, region); }

    long info(unsigned long *size, unsigned long *flags)
    { return SimplePager::info(size, flags); }
};

// vim: tabstop=4 expandtab shiftwidth=4
