/*
 * A resource offering support for creating contexts and opening files.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/ipc.h>

#include "file_paging.h"
#include "opener_context_resource.h"
#include "opener_interface.h"
#include "pages.h"
#include "resource.h"



/* Support for providing access to files. */

class OpenerResource : public Resource, public FilePaging, public Opener
{
public:
    explicit OpenerResource(Pages *pages);

    /* Server details. */

    int expected_items();

    ipc_server_handler_type handler();

    void *interface()
    { return static_cast<Opener *>(this); }

    /* Direct access methods. */

    Pager *open(const char *path, flags_t flags);

    /* Opener interface methods. */

    long context(l4_cap_idx_t *context);
};

// vim: tabstop=4 expandtab shiftwidth=4
